﻿using System.Reflection;

using Mono.Cecil;
using Mono.Cecil.Cil;

class Program {
	const string OriginalDll = "HelloLibrary.dll";
	const string PatchedAssemblyName = "HelloLibraryPatched";
	const string PatchedDll = PatchedAssemblyName + ".dll";

	static void Main(string[] args) {
		//
		// Mono.Cecil & Mono.Cecil.Cil
		//

		AssemblyDefinition asmDef = AssemblyDefinition.ReadAssembly(OriginalDll);
		TypeDefinition typeDef = asmDef.MainModule.GetType("HelloString");
		MethodDefinition? getItMethodDef = null;
		foreach (var m in typeDef.Methods) {
			if (m.Name == "GetIt") {
				getItMethodDef = m;
				break;
			}
		}

		if (getItMethodDef is not null) {
			ILProcessor ilProc = getItMethodDef.Body.GetILProcessor();

			getItMethodDef.Body.Instructions.Clear();

			ilProc.Append(ilProc.Create(OpCodes.Ldstr, "No nazdar!"));
			ilProc.Append(ilProc.Create(OpCodes.Ret));

			asmDef.Name.Name = PatchedAssemblyName; // Name change is necessary, otherwise the Assembly.LoadFrom("HelloLibraryPatched.dll") will pickup the already loaded HelloLibrary.dll, as both will have the same assembly name without the change! If we included a post-build step to copy HelloLibrary to AppNet7 folder, and did not reference the HelloLibrary from AppNet7 project, this step would not be necessary.
			asmDef.Write(PatchedDll);
		}

		//
		// System.Reflection
		//

		Console.Write("What to load (p = patched, o = original): ");
		string? choice = Console.ReadLine();

		string asmName = choice == "p" ? PatchedDll : OriginalDll;

		Assembly a = Assembly.LoadFrom(asmName);
		IHelloProvider? hp = (IHelloProvider?) a.CreateInstance("CoolHelloProvider");
		hp?.SayHello();
	}
}